﻿using System;
using System.Collections.Generic;
using System.Data.Common;
using System.Linq;
using System.Text;
using Telerik.OpenAccess.Data;

namespace StringToBoolSln
{
    class Program
    {
        static void Main(string[] args)
        {
            var context = new EntityDiagrams1();
            var model = context.Metadata;

            context.TestTables.ToList().ForEach(f => Console.WriteLine(f.BoolVal));
        }
    }

    internal class StringToBoolConverter : AdoTypeConverter
    {
        private bool nullable;

        public override AdoTypeConverter Initialize(IDataColumn user, Type clr, IAdoTypeConverterRegistry registry, bool secondary)
        {
            if (clr == typeof(bool) || clr == typeof(bool?))
            {
                nullable = (clr == typeof(bool?));
                //paramSql = registry.GetParameterSql(user, clr);
                return base.Initialize(user, clr, registry, secondary);
            }

            return base.Initialize(user, clr, registry, secondary);
        }

        public override object Read(ref DataHolder data)
        {
            data.NoValue = data.Reader.IsDBNull(data.Position);
            
            if (data.NoValue == false)
            {
                data.BooleanValue = false;//your default here!
                string value = data.Reader.GetValue(data.Position).ToString();
                bool boolValue = false;
                if (bool.TryParse(value, out boolValue))
                {
                    data.BooleanValue = boolValue;
                }
            }
            else 
            {
                if (nullable)
                    data.ObjectValue = null;
                else
                    data.BooleanValue = false; //your default!
            }

            return data.ObjectValue ?? data.BooleanValue;
        }

        public override void Write(ref DataHolder data)
        {
            DbParameter p = data.Parameter;
            p.DbType = System.Data.DbType.AnsiString;
            p.Size = 5;
            if (nullable)
                p.Value = data.ObjectValue ?? false;//default
            else
                p.Value = data.BooleanValue.ToString();

            
        }

        public override bool CreateLiteralSql(ref DataHolder data)
        {
            if (data.NoValue || data.StringValue == null)
            {
                data.StringValue = "NULL";
                return false;
            }
            else
            {
                return true;
            }
        }

        public override Type DefaultType { get { return typeof(bool?); } }
    }
}
